#include "c64.h"
#include "g642d64.h"			// g64 conversion to d64
#include <string.h>				// memcpy

#include "c_D64.h"				// Standard disk
#include "c_CRT.h"				// CCS64
#include "c_PRG.h"				// Raw Binary
#include "c_T64.h"				// Binary tape (TAP is pulse-encoded)
#include "c_Pxx.h"				// PC64

char C64HaveFile[] = "C64have.dat";
char C64DatFile[] = "C64.dat";
char *C64Ext[10] = {					// null entry, for returning NULL on not-found
	"????\0",	".D64\0",	".T64\0",	".PRG\0",
	".P00\0",	".CRT\0",	".G64\0",	".TAP\0",
	".ZIP\0",	"\0"
//	"\0"	|	"\0\0\0\0\0"	|	"\0x00"
};

unsigned C64Unk( char *image, unsigned size) {
	char cMagic_crt[] = "C64 CARTRIDGE";
	char cMagic_pxx[] = "C64File";
	char cMagic_tap[] = "C64-TAPE-RAW";
	char cMagic_t64[] = "C64";
		// t64 could be "C64 tape image file"
		// or "C64S tape file" or "C64s tape image file"...
		// so we save it for last

	if(image[0] == 1 && image[1] == 8)
		return TYPE_PRG;		// a few have different load addresses
	if(!strncmp(image, cMagic_crt, strlen(cMagic_crt)))
		return TYPE_CRT;
	if(!strncmp(image, cMagic_pxx, strlen(cMagic_pxx)))
		return TYPE_Pxx;
	if(!strncmp(image, cMagic_tap, strlen(cMagic_tap)))
		return TYPE_TAP;
	if(!strncmp(image, cMagic_t64, strlen(cMagic_t64)))
		return TYPE_T64;
	// d64 test checks for minimum size of 18 tracks + 1 sector)
	// 0x16500 is start of BAM which *should* point to dir (track 18 sector 1)
	if(size > 0x16699 && (image[0x16500] == 0x12 && image[0x16501] == 0x01))
		return TYPE_D64;
	return 0;
}

CImage *C64ClassOf( int index, char *image, unsigned size ) {
	CImage *ciTmp;
	char *cnv = 0;						// compiler whines if it is under TYPE_G64

	if(index < 0)
		index = C64Unk( image, size);

	switch( index ) {
	case TYPE_D64:
		ciTmp = new CImageD64(image, size) ;
		break;
	case TYPE_T64:
		ciTmp = new CImageT64(image, size);
		break;
	case TYPE_PRG:
		ciTmp = new CImagePRG(image, size);
		break;
	case TYPE_Pxx:
		ciTmp = new CImagePxx(image, size);
		break;
	case TYPE_CRT:
		ciTmp = new CImageCRT(image, size);
		break;
	case TYPE_TAP:
		ciTmp = new CImagePRG(image, size);	// Temp Hack for TAP
		break;
	case TYPE_G64: // G64 converted to D64
		cnv = new char[size];
		size = g642d64((unsigned char *)image, (unsigned char *) cnv);
		memcpy(image, cnv, size);
		delete [] cnv;
		ciTmp = new CImageD64(image, size) ;
		break;
	default:
		ciTmp = 0;
	};
	return ciTmp;
}
